# Generated by Django 3.2.13 on 2022-07-04 15:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_lab", "0022_auto_20211210_1839"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="historicalaliquot",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Aliquot",
                "verbose_name_plural": "historical Aliquots",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalbox",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Box",
                "verbose_name_plural": "historical Boxes",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalboxitem",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Box Item",
                "verbose_name_plural": "historical Box Items",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalconsignee",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Consignee",
                "verbose_name_plural": "historical Consignees",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalmanifest",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Manifest",
                "verbose_name_plural": "historical Manifests",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalorder",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Order",
                "verbose_name_plural": "historical Orders",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalresult",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Result",
                "verbose_name_plural": "historical Results",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalresultitem",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Result Item",
                "verbose_name_plural": "historical Result Items",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalshipper",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Shipper",
                "verbose_name_plural": "historical Shippers",
            },
        ),
        migrations.AlterField(
            model_name="historicalaliquot",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalbox",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalboxitem",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalconsignee",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalmanifest",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalorder",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalresult",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalresultitem",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalshipper",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
    ]
