CONDITION_OK = "OK"
FILL_ACROSS = "across"
FILL_DOWN = "down"
FINGER_PRICK = "finger_prick"
PACKED = "packed"
SHIPPED = "shipped"
STORAGE = "storage"
TESTING = "testing"
TUBE = "tube"
VERIFIED = "verified"
WHOLE_BLOOD = "02"
EQ = "="
GT = ">"
GTE = ">="
LT = "<"
LTE = "<="
